@@ -1,4 +1,5 @@
 # Copyright 2010-2011 OpenStack Foundation
+# Copyright 2018 Verizon Wireless
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import logging
 
 from oslo_config import cfg
@@ -26,7 +28,6 @@ from glance_store import exceptions
 from glance_store.i18n import _
 from glance_store import location
 
-
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
@@ -35,15 +36,15 @@ _STORE_OPTS = [
                 default=['file', 'http'],
                 deprecated_for_removal=True,
                 deprecated_since='Rocky',
-                deprecated_reason=_("""
+                deprecated_reason="""
 This option is deprecated against new config option
 ``enabled_backends`` which helps to configure multiple backend stores
 of different schemes.
 
 This option is scheduled for removal in the Stein development
 cycle.
-"""),
-                help=_("""
+""",
+                help="""
 List of enabled Glance stores.
 
 Register the storage backends to use for storing disk images
@@ -63,24 +64,23 @@ Possible values:
 Related Options:
     * default_store
 
-""")),
+"""),
     cfg.StrOpt('default_store',
                default='file',
                choices=('file', 'filesystem', 'http', 'https', 'swift',
                         'swift+http', 'swift+https', 'swift+config', 'rbd',
                         'sheepdog', 'cinder', 'vsphere'),
-
                deprecated_for_removal=True,
                deprecated_since='Rocky',
-               deprecated_reason=_("""
+               deprecated_reason="""
 This option is deprecated against new config option
 ``default_backend`` which acts similar to ``default_store`` config
 option.
 
 This option is scheduled for removal in the Stein development
 cycle.
-"""),
-               help=_("""
+""",
+               help="""
 The default scheme to use for storing images.
 
 Provide a string value representing the default scheme to use for
@@ -108,21 +108,21 @@ Possible values:
 Related Options:
     * stores
 
-""")),
+"""),
     cfg.IntOpt('store_capabilities_update_min_interval',
                default=0,
                min=0,
                deprecated_for_removal=True,
                deprecated_since='Rocky',
-               deprecated_reason=_("""
+               deprecated_reason="""
 This option configures a stub method that has not been implemented
 for any existing store drivers.  Hence it is non-operational, and
 giving it a value does absolutely nothing.
 
 This option is scheduled for removal early in the Stein development
 cycle.
-"""),
-               help=_("""
+""",
+               help="""
 Minimum interval in seconds to execute updating dynamic storage
 capabilities based on current backend status.
 
@@ -158,7 +158,7 @@ Possible values:
 Related Options:
     * None
 
-""")),
+"""),
 ]
 
 _STORE_CFG_GROUP = 'glance_store'
@@ -438,6 +438,25 @@ def check_location_metadata(val, key=''):
                                           % dict(key=key, type=type(val)))
 
 
+def _check_metadata(store, metadata):
+    if not isinstance(metadata, dict):
+        msg = (_("The storage driver %(driver)s returned invalid "
+                 " metadata %(metadata)s. This must be a dictionary type")
+               % dict(driver=str(store), metadata=str(metadata)))
+        LOG.error(msg)
+        raise exceptions.BackendException(msg)
+    try:
+        check_location_metadata(metadata)
+    except exceptions.BackendException as e:
+        e_msg = (_("A bad metadata structure was returned from the "
+                   "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
+                 dict(driver=encodeutils.exception_to_unicode(store),
+                      metadata=encodeutils.exception_to_unicode(metadata),
+                      e=encodeutils.exception_to_unicode(e)))
+        LOG.error(e_msg)
+        raise exceptions.BackendException(e_msg)
+
+
 def store_add_to_backend(image_id, data, size, store, context=None,
                          verifier=None):
     """
@@ -461,25 +480,49 @@ def store_add_to_backend(image_id, data, size, store, context=None,
                                                      context=context,
                                                      verifier=verifier)
     if metadata is not None:
-        if not isinstance(metadata, dict):
-            msg = (_("The storage driver %(driver)s returned invalid "
-                     " metadata %(metadata)s. This must be a dictionary type")
-                   % dict(driver=str(store), metadata=str(metadata)))
-            LOG.error(msg)
-            raise exceptions.BackendException(msg)
-        try:
-            check_location_metadata(metadata)
-        except exceptions.BackendException as e:
-            e_msg = (_("A bad metadata structure was returned from the "
-                       "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
-                     dict(driver=encodeutils.exception_to_unicode(store),
-                          metadata=encodeutils.exception_to_unicode(metadata),
-                          e=encodeutils.exception_to_unicode(e)))
-            LOG.error(e_msg)
-            raise exceptions.BackendException(e_msg)
+        _check_metadata(store, metadata)
+
     return (location, size, checksum, metadata)
 
 
+def store_add_to_backend_with_multihash(
+        image_id, data, size, hashing_algo, store,
+        context=None, verifier=None):
+    """
+    A wrapper around a call to each store's add() method that requires
+    a hashing_algo identifier and returns a 5-tuple including the
+    "multihash" computed using the specified hashing_algo.  (This
+    is an enhanced version of store_add_to_backend(), which is left
+    as-is for backward compatibility.)
+
+    :param image_id:  The image add to which data is added
+    :param data: The data to be stored
+    :param size: The length of the data in bytes
+    :param store: The store to which the data is being added
+    :param hashing_algo: A hashlib algorithm identifier (string)
+    :param context: The request context
+    :param verifier: An object used to verify signatures for images
+    :return: The url location of the file,
+             the size amount of data,
+             the checksum of the data,
+             the multihash of the data,
+             the storage system's metadata dictionary for the location
+    :raises: ``glance_store.exceptions.BackendException``
+             ``glance_store.exceptions.UnknownHashingAlgo``
+    """
+
+    if hashing_algo not in hashlib.algorithms_available:
+        raise exceptions.UnknownHashingAlgo(algo=hashing_algo)
+
+    (location, size, checksum, multihash, metadata) = store.add(
+        image_id, data, size, hashing_algo, context=context, verifier=verifier)
+
+    if metadata is not None:
+        _check_metadata(store, metadata)
+
+    return (location, size, checksum, multihash, metadata)
+
+
 def add_to_backend(conf, image_id, data, size, scheme=None, context=None,
                    verifier=None):
     if scheme is None:
