@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import logging
 
 from oslo_config import cfg
@@ -23,7 +24,7 @@ from stevedore import extension
 
 from glance_store import capabilities
 from glance_store import exceptions
-from glance_store.i18n import _, _LW
+from glance_store.i18n import _
 from glance_store import location
 
 
@@ -123,7 +124,7 @@ def _list_driver_opts():
 
 
 def register_store_opts(conf):
-    LOG.debug("Registering options for group %s" % _STORE_CFG_GROUP)
+    LOG.debug("Registering options for group %s", _STORE_CFG_GROUP)
     conf.register_opts(_STORE_OPTS, group=_STORE_CFG_GROUP)
 
     driver_opts = _list_driver_opts()
@@ -133,7 +134,7 @@ def register_store_opts(conf):
             if enabled_backends[backend] not in opt_list:
                 continue
 
-            LOG.debug("Registering options for group %s" % backend)
+            LOG.debug("Registering options for group %s", backend)
             conf.register_opts(driver_opts[opt_list], group=backend)
 
 
@@ -153,7 +154,7 @@ def _load_multi_store(conf, store_entry,
         return mgr.driver
     except RuntimeError as e:
         LOG.warning("Failed to load driver %(driver)s. The "
-                    "driver will be disabled" % dict(driver=str([driver, e])))
+                    "driver will be disabled", dict(driver=str([driver, e])))
 
 
 def _load_multi_stores(conf):
@@ -176,10 +177,7 @@ def _load_multi_stores(conf):
 
 
 def create_multi_stores(conf=CONF):
-    """
-    Registers all store modules and all schemes
-    from the given config.
-    """
+    """Registers all store modules and all schemes from the given config."""
     store_count = 0
     scheme_map = {}
     for (store_entry, store_instance,
@@ -191,9 +189,9 @@ def create_multi_stores(conf=CONF):
             continue
 
         if not schemes:
-            raise exceptions.BackendException('Unable to register store %s. '
-                                              'No schemes associated with it.'
-                                              % store_entry)
+            raise exceptions.BackendException(
+                _('Unable to register store %s. No schemes associated '
+                  'with it.') % store_entry)
         else:
             LOG.debug("Registering store %s with schemes %s",
                       store_entry, schemes)
@@ -227,7 +225,8 @@ def verify_store():
 
 
 def get_store_from_store_identifier(store_identifier):
-    """
+    """Determine backing store from identifier.
+
     Given a store identifier, return the appropriate store object
     for handling that scheme.
     """
@@ -280,11 +279,30 @@ def add(conf, image_id, data, size, backend, context=None,
                                 verifier)
 
 
+def _check_metadata(store, metadata):
+    if not isinstance(metadata, dict):
+        msg = (_("The storage driver %(driver)s returned invalid "
+                 " metadata %(metadata)s. This must be a dictionary type")
+               % dict(driver=str(store), metadata=str(metadata)))
+        LOG.error(msg)
+        raise exceptions.BackendException(msg)
+    try:
+        check_location_metadata(metadata)
+    except exceptions.BackendException as e:
+        e_msg = (_("A bad metadata structure was returned from the "
+                   "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
+                 dict(driver=encodeutils.exception_to_unicode(store),
+                      metadata=encodeutils.exception_to_unicode(metadata),
+                      e=encodeutils.exception_to_unicode(e)))
+        LOG.error(e_msg)
+        raise exceptions.BackendException(e_msg)
+
+
 def store_add_to_backend(image_id, data, size, store, context=None,
                          verifier=None):
-    """
-    A wrapper around a call to each stores add() method.  This gives glance
-    a common place to check the output
+    """A wrapper around a call to each stores add() method.
+
+    This gives glance a common place to check the output.
 
     :param image_id:  The image add to which data is added
     :param data: The data to be stored
@@ -305,25 +323,49 @@ def store_add_to_backend(image_id, data, size, store, context=None,
                                                      verifier=verifier)
 
     if metadata is not None:
-        if not isinstance(metadata, dict):
-            msg = (_("The storage driver %(driver)s returned invalid "
-                     " metadata %(metadata)s. This must be a dictionary type")
-                   % dict(driver=str(store), metadata=str(metadata)))
-            LOG.error(msg)
-            raise exceptions.BackendException(msg)
-        try:
-            check_location_metadata(metadata)
-        except exceptions.BackendException as e:
-            e_msg = (_("A bad metadata structure was returned from the "
-                       "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
-                     dict(driver=encodeutils.exception_to_unicode(store),
-                          metadata=encodeutils.exception_to_unicode(metadata),
-                          e=encodeutils.exception_to_unicode(e)))
-            LOG.error(e_msg)
-            raise exceptions.BackendException(e_msg)
+        _check_metadata(store, metadata)
+
     return (location, size, checksum, metadata)
 
 
+def store_add_to_backend_with_multihash(
+        image_id, data, size, hashing_algo, store,
+        context=None, verifier=None):
+    """
+    A wrapper around a call to each store's add() method that requires
+    a hashing_algo identifier and returns a 5-tuple including the
+    "multihash" computed using the specified hashing_algo.  (This
+    is an enhanced version of store_add_to_backend(), which is left
+    as-is for backward compatibility.)
+
+    :param image_id:  The image add to which data is added
+    :param data: The data to be stored
+    :param size: The length of the data in bytes
+    :param store: The store to which the data is being added
+    :param hashing_algo: A hashlib algorithm identifier (string)
+    :param context: The request context
+    :param verifier: An object used to verify signatures for images
+    :return: The url location of the file,
+             the size amount of data,
+             the checksum of the data,
+             the multihash of the data,
+             the storage system's metadata dictionary for the location
+    :raises: ``glance_store.exceptions.BackendException``
+             ``glance_store.exceptions.UnknownHashingAlgo``
+    """
+
+    if hashing_algo not in hashlib.algorithms_available:
+        raise exceptions.UnknownHashingAlgo(algo=hashing_algo)
+
+    (location, size, checksum, multihash, metadata) = store.add(
+        image_id, data, size, hashing_algo, context=context, verifier=verifier)
+
+    if metadata is not None:
+        _check_metadata(store, metadata)
+
+    return (location, size, checksum, multihash, metadata)
+
+
 def check_location_metadata(val, key=''):
     if isinstance(val, dict):
         for key in val:
@@ -349,9 +391,8 @@ def delete(uri, backend, context=None):
         store = get_store_from_store_identifier(backend)
         return store.delete(loc, context=context)
 
-    msg = _LW('Backend is not set to image, searching '
-              'all backends based on location URI.')
-    LOG.warn(msg)
+    LOG.warning('Backend is not set to image, searching all backends based on '
+                'location URI.')
 
     backends = CONF.enabled_backends
     for backend in backends:
@@ -400,9 +441,8 @@ def get(uri, backend, offset=0, chunk_size=None, context=None):
                          chunk_size=chunk_size,
                          context=context)
 
-    msg = _LW('Backend is not set to image, searching '
-              'all backends based on location URI.')
-    LOG.warn(msg)
+    LOG.warning('Backend is not set to image, searching all backends based on '
+                'location URI.')
 
     backends = CONF.enabled_backends
     for backend in backends:
